package xena;

import xena.XenaCom.XenaLatencyStatistics;
import xena.XenaCom.XenaStatistics;

public interface XenaAPI {
	// ======================================= ENUMERATIONS ====================================
	/*public static enum duplexStates {
		Full,
		Half
	}*/
	
	public static enum portRates {
		Rate_10M(10),
		Rate_100M(100),
		Rate_1G(1000);
		
		int rateMbps;
		portRates(int _rateMbps) {
			rateMbps = _rateMbps;
		}
		public int getRateMbps() {
			return rateMbps;
		}
		static portRates toRate(int _rate) throws Exception {
			portRates theRate;
			
			switch (_rate) {
			case 10:
				theRate = Rate_10M;
				break;
			case 100:
				theRate = Rate_100M;
				break;
			case 1000:
				theRate = Rate_1G;
				break;
			default:
				throw new Exception(String.format("Unsupported rate %dMbps!", _rate));
			}
			return theRate;
		}
	}
	
	public static enum streamStates {
		Enabled(true),
		Disabled(false),
		Suppressed(false);
		
		boolean enabled;
		streamStates(boolean _enabled) {
			enabled = _enabled;
		}
		/**
		 * @return the enabled
		 */
		public boolean isEnabled() {
			return enabled;
		}
		/**
		 * @param enabled the enabled to set
		 */
		public void setEnabled(boolean enabled) {
			this.enabled = enabled;
		}
	}
	
	public static enum streamRateTypes {
		Percent(44, "Percent"),
		Mbps(100, "Mbps"),
		Pps(10000, "PPS");
		
		int defaultVal;
		String description;
		streamRateTypes(int val, String _description) {
			defaultVal = val;
			description = _description;
		}
		
		public int getValue() {
			return defaultVal;
		}

		/**
		 * @return the description
		 */
		public String getDescription() {
			return description;
		}

	}
	public static enum defaultTrafficTypes {
		Tagged,
		Untagged
	}
	
	// =========================================================================================
	
	// ======================================= APIs ============================================
	/**
	 * Configures a port for a specific rate, auto-negotiation state and a given description.
	 *
	 * @param  params a structure containing all parameters relevant for port configuration
	 * @return       a port configuration response structure which contains indications as to the port's configuration success and sync status
	 */
	public portConfigurationResponse configurePort(portConfigurationParams params) throws Exception;
	
	/**
	 * Get  port actual speed in terms of Etherhaul (10fd, 100fd, 1000fd)
	 *
	 * @param   logicalPortAddress the logical address of the port on which the stream is configured
	 * @return       a port speed configuration response (10,100,1000)
	 */
	public String portSpeedStatus(int logicalPortAddress) throws Exception;

	/**
	 * Creates a stream on a given logical port. The created stream will not be transmitted
	 * until the port's traffic state is enabled
	 *
	 * @param  params a structure containing all parameters relevant for stream creation (including headers and modifiers)
	 * @return       a port configuration response structure which indicates whether the stream was created successfully, as well as its ID
	 */
	public streamCreationResponse createStream(streamCreationParams params) throws Exception;
	
	/**
	 * Sets a stream's state to enabled, disabled or suppressed
	 *
	 * @param   logicalPortAddress the logical address of the port on which the stream is configured
	 * @param	streamID the ID of the stream
	 * @param	state the state of the stream - enabled, disabled or suppressed (enum)
	 * @return       whether the stream's state was modified successfully
	 */
	public boolean setStreamState(int logicalPortAddress, int streamID, streamStates state) throws Exception;
	
	/**
	 * Retrieve a port's sync status (synced / non-synced)
	 *
	 * @param   logicalPortAddress the logical address of the port on which the stream is configured
	 * @return       whether the port is synced (true) or not (false)
	 */
	public boolean isPortSynced(int logicalPortAddress) throws Exception;
	
	/**
	 * Retrieve a port's sync status (up / down) same as isPortSynced only return different return values
	 *
	 * @param   logicalPortAddress the logical address of the port on which the stream is configured
	 * @return       whether the port is synced (up) or not (down)
	 */
	public String isPortSyncedStatus(int logicalPortAddress) throws Exception;

	
	public boolean isXenaEnabled() throws Exception;
	/**

	 * Injects FCS errors into a given stream
	 *
	 * @param   streamID the ID of the stream on which to inject FCS errors 
	 * @return       whether the operation completed successfully
	 */
	public void injectErrorsIntoStreamCheckSum(/*int logicalPortAddress, */int streamID, int N) throws Exception;
	
	/**
	 * Resets the Xena chassis, removing all statistics and streams
	 *
	 */
	public void reset()  throws Exception;

	/**
	 * Removes all streams from the chassis
	 *
	 */
	public void removeAllStreams() throws Exception;

	/**
	 * Returns a stream object, pertaining to a configured stream
	 *
	 * @param   index the index of the stream to return 
	 * @return       a XenaStream object 
	 */
	public XenaStream getStream(int index);
	
	/**
	 * Starts traffic of all enabled streams, waits for a predefined period then stops the traffic
	 *
	 * @param   msec the time period, in ms, in which traffic will be active 
	 */
	public void start(int msec) throws Exception;

	/**
	 * Starts traffic of all enabled streams, waits for a predefined period then stops the traffic.
	 * During the traffic's run, the packets on each port will be captured to a valid pcap file
	 *
	 * @param   msec the time period, in ms, in which traffic will be active 
	 * 			captureTraffic a flag indicating whether to capture the traffic on all used ports
	 * 			capturePath the path where to save the captured data to 
	 */
	public void start(int msec, boolean captureTraffic, String capturePath) throws Exception;
	
	/**
	 * Starts traffic of all enabled streams and lets it run indefinitely
	 *
	 */
	
	public void start() throws Exception;
	/**
	 * Starts traffic of all enabled streams and lets it run indefinitely. The packets will be captured to a valid pcap file;
	 *
	 */
	
	public void start(Boolean capture) throws Exception;
	public void start1(Boolean capture) throws Exception; //yair - start only capture
	
	
	/**
	 * Stops all active traffic and finishes capture;
	 *
	 */
	public void stop() throws Exception;
	/**
	 * Stops all active traffic
	 *
	 */
	public void stop(Boolean capture, String capturePath) throws Exception;
	public void stop1(Boolean capture, String capturePath) throws Exception; //yair - stops only capture
	
	
	/**
	 * Reads the TX statistics of a given logical port
	 *
	 * @param   logicalPortAddress the logical address of the port 
	 * @return       a XenaStatistics structure containing the TX statistics
	 */
	public XenaStatistics getPortTxStatistcs(int logicalPortAddress) throws Exception;
	
	/**
	 * Reads the RX statistics of a given logical port
	 *
	 * @param   logicalPortAddress the logical address of the port 
	 * @return       a XenaStatistics structure containing the Rx statistics
	 */	
	public XenaStatistics getPortRxStatistcs(int logicalPortAddress) throws Exception;
	
	 /**
	 * Clears all statistics on a given logical port
	 *
	 * @param   logicalPortAddress the logical address of the port 
	 */	public void clearPortStatistics(int logicalPortAddress) throws Exception;
	
	 /**
		 * Clears the statistics of all ports
		 *
		 */	
	public void clearAllStatistics() throws Exception;
	
	/**
	 * Reads the RX statistics of a given stream 
	 *
	 * @param   logicalPortAddress the logical address of the port 
	 * @return       a XenaStatistics structure containing the Rx statistics
	 */	
	public streamRxStatistics getStreamRxStatistics(int streamID) throws Exception;
	
	/**
	 * Reads the RX statistics of a given stream 
	 *
	 * @param   streamID the ID of the stream 
	 * @return       a XenaStatistics structure containing the Rx statistics
	 */	
	public XenaStatistics getRxStatistics(int streamID) throws Exception;
	
	/**
	 * Reads the RX latency statistics of a given stream 
	 *
	 * @param   streamID the ID of the stream 
	 * @return       a XenaLatencyStatistics structure 
	 */	
	public XenaLatencyStatistics getRxLatencyStatistics(int streamID) throws Exception;
	
	/**
	 * Returns the combined latency (minimal min, maximal max and averge) of all streams enabled on the device 
	 *
	 * @return       a XenaLatencyStatistics structure 
	 */	
	public XenaLatencyStatistics getLatencyStatistics() throws Exception;

	/**
	 * Reads the Tx statistics of a given stream 
	 *
	 * @param   logicalPortAddress the logical address of the port 
	 * @param   streamID the ID of the stream 
	 * @return       a XenaLatencyStatistics structure 
	 */	
	public streamTxStatistics getStreamTxStatistics(int logicalPortAddress, int streamID) throws Exception;

	/**
	 * Injects N FCS errors into the stream 
	 *
	 * @param   streamID the ID of the stream 
	 * @param   numberOfErrors the number of packets to inject errors to 
	 * @return       none 
	 */	
	public void injectErrorsIntoStreamTPLD(int streamID, int N) throws Exception;
	
	public int getStreamCount();

	/**
	 * Captures traffic on a port to a specified pcap file 
	 *
	 * @param   logicalPortAddress the logical address of the port 
	 * @param   fileName the full path of the capture file 
	 * @return       Whether the operation completed successfully 
	 */
	public boolean capturePortTrafficToFile(int logicalPortAddress, String captureFileFullPath) throws Exception;
	// =========================================================================================	

	public boolean playbackCaptureFile(int logicalPortAddress, String captureFileFullPath) throws Exception;
	public void setCaptureLength(int numberOfPacketsToCapture) throws Exception;
	public void setCapturePathForThread(String capturePath);
}
